from __future__ import absolute_import

from procstream.source.source import DataSourceService
from procstream.source.source_twitter import TwitterDataCollector

__all__ = ['DataSourceService', 'TwitterDataCollector']


import os, json
from procstream import DataSourceService
import logging as logger

new_config = {"FILE_NAME": os.environ.get("FILE_NAME", "")}


class FileReader():

    def __init__(self, datasource_service_obj, file_name):
        self.datasource_service_obj = datasource_service_obj
        self.file_name = file_name

    def file_data_producer(self):
        logger.info("Reading File and producing to Kafka...")
        with open(self.file_name, 'r') as reader:
            for line in reader.readlines():
                self.datasource_service_obj.produce_to_topic(line)


def main():
    datasource_service_obj = DataSourceService(new_config)

    file_reader = FileReader(datasource_service_obj, new_config.get("FILE_NAME"))

    file_reader.file_data_producer()



from unittest import TestCase
from procstream import StreamProcessMicroService


class StreamProcessTestImpl(StreamProcessMicroService):
    def process_message(self, message):
        print(message)
        return message


class Test(TestCase):

    def setUp(self) -> None:
        config = {"KAFKA_TARGET_BOOTSTRAP_SERVERS": "localhost",
                  "KAFKA_SOURCE_BOOTSTRAP_SERVERS": "localhost",
                  "KAFKA_SOURCE_TOPIC": "topic1",
                  "KAFKA_TARGET_TOPIC": "topic2",
                  "CONSUMER_GROUP": "cg01",
                  "MODULE_NAME": 'test_module'}

        self.k_service = StreamProcessTestImpl(config)

    def test_kafka_producer_connection(self):
        self.assertTrue(self.k_service.get_producer_client().bootstrap_connected())

    def test_kafka_consumer_connection(self):
        self.assertTrue(self.k_service.get_consumer_client().bootstrap_connected())



import os, json
from tweepy.streaming import StreamListener
from tweepy import OAuthHandler
from tweepy import Stream
from procstream.source.source import DataSourceService


config = {"TWITTER_ACCESS_TOKEN": os.environ.get("TWITTER_ACCESS_TOKEN", ""),
          "TWITTER_ACCESS_TOKEN_SECRET": os.environ.get("TWITTER_ACCESS_TOKEN_SECRET", ""),
          "TWITTER_CONSUMER_KEY": os.environ.get("TWITTER_CONSUMER_KEY", ""),
          "TWITTER_CONSUMER_SECRET": os.environ.get("TWITTER_CONSUMER_SECRET", ""),
          "TWITTER_FILTER_TRACK": os.environ.get("TWITTER_FILTER_TRACK", ""),
          "MODULE_NAME": os.environ.get("MODULE_NAME", "TWITTER_DATA_COLLECTOR"),
          "CONSUMER_GROUP": os.environ.get("CONSUMER_GROUP", "TWITTER_DATA_COLLECTOR_CG")
          }


class StdOutListener(StreamListener):

    def __init__(self, datasource_service_obj):
        super().__init__()
        self.datasource_service_obj = datasource_service_obj

    def on_data(self, data):
        # print(data)
        tweet_data = json.loads(data)
        for key, value in tweet_data.items():
            if isinstance(value, str):
                new_string = value.replace("\r", " ")
                tweet_data[key] = new_string.replace("\n", " ")
        self.datasource_service_obj.produce_to_topic(tweet_data)
        print(tweet_data)
        return True

    def on_error(self, status):
        print(status)


class TwitterDataCollector():

    def run(self, new_config=None):
        new_config = config if not new_config else new_config

        twitter_data_source_service_obj = DataSourceService(new_config)

        listener = StdOutListener(twitter_data_source_service_obj)

        auth = OAuthHandler(new_config.get("TWITTER_CONSUMER_KEY"), new_config.get("TWITTER_CONSUMER_SECRET"))

        auth.set_access_token(new_config.get("TWITTER_ACCESS_TOKEN"), new_config.get("TWITTER_ACCESS_TOKEN_SECRET"))

        stream = Stream(auth, listener)

        stream.filter(track=new_config.get("TWITTER_FILTER_TRACK").split(","))



from abc import ABC, abstractmethod
from kafka import KafkaProducer
from kafka import KafkaConsumer
import logging as logger
import json
import os

logger.basicConfig(format='%(asctime)s|[%(levelname)s]|File:%(filename)s|'
                          'Function:%(funcName)s|Line:%(lineno)s|%(message)s')


default_config = {"KAFKA_SOURCE_BOOTSTRAP_SERVERS": os.environ.get("KAFKA_SOURCE_BOOTSTRAP_SERVERS",
                                                          "localhost:9092"),
                  "KAFKA_SOURCE_TOPIC": os.environ.get('KAFKA_SOURCE_TOPIC', ''),
                  "KAFKA_TARGET_BOOTSTRAP_SERVERS": os.environ.get("KAFKA_TARGET_BOOTSTRAP_SERVERS",
                                                          "localhost:9092"),
                  "KAFKA_TARGET_TOPIC": os.environ.get('KAFKA_TARGET_TOPIC', ''),
                  "MODULE_NAME": os.environ.get('MODULE_NAME', ''),
                  "CONSUMER_GROUP": os.environ.get("CONSUMER_GROUP", '')}

""" Base Class for pipeline module """


class StreamProcessMicroService(ABC):

    @staticmethod
    def forgiving_json_deserializer(v):
        if v is None:
            return
        try:
            return json.loads(v.decode('utf-8'))
        except json.decoder.JSONDecodeError:
            logger.error('Unable to decode: %s', v)
            return None

    def __init__(self, new_config):
        self.config = {**default_config, **new_config}
        self.verify_env()
        logger.info("Connecting to Kafka Consumer bootstrap server")
        self.consumer_client = KafkaConsumer(self.config.get("KAFKA_SOURCE_TOPIC"), group_id=self.config.get("CONSUMER_GROUP"),
                                             bootstrap_servers=self.config.get("KAFKA_SOURCE_BOOTSTRAP_SERVERS").split(","),
                                             value_deserializer=lambda v: self.forgiving_json_deserializer(v))
        logger.info("Connecting to Kafka Producer bootstrap server")
        self.producer_client = KafkaProducer(bootstrap_servers=self.config.get("KAFKA_TARGET_BOOTSTRAP_SERVERS").split(","),
                                             value_serializer=lambda v: json.dumps(v).encode('utf-8'))
        self.target_topic = self.config.get("KAFKA_TARGET_TOPIC")

    def verify_env(self):
        verification_failed = False
        for key, value in self.config.items():
            if not str.strip(value):
                verification_failed = True
                logger.error(f"Environment variable '{key}' not set.")
        if verification_failed:
            quit(1)

    def get_producer_client(self):
        return self.producer_client

    def get_consumer_client(self):
        return self.consumer_client

    @abstractmethod
    def process_message(self, message):
        """ Abstract Method. Override this to process the message """
        raise NotImplementedError('Implement me in subclass')

    def kafka_pipeline_service(self):
        """ Skeleton of operations to perform. DON'T override """
        print("Starting consumer")
        for message in self.consumer_client:
            processed_message = self.process_message(message)
            if processed_message:
                self.producer_client.send(self.target_topic, processed_message)

    def start_service(self):
        self.kafka_pipeline_service()



from abc import ABC, abstractmethod
from kafka import KafkaProducer
from kafka import KafkaConsumer
import logging as logger
import json
import os

default_config = {"KAFKA_TARGET_BOOTSTRAP_SERVERS": os.environ.get("KAFKA_TARGET_BOOTSTRAP_SERVERS",
                                                                   "localhost:9092"),
                  "KAFKA_TARGET_TOPIC": os.environ.get("KAFKA_TARGET_TOPIC", ""),
                  "MODULE_NAME": os.environ.get("MODULE_NAME", ""),
                  "CONSUMER_GROUP": os.environ.get("CONSUMER_GROUP", "")}


class DataSourceService(ABC):
    def __init__(self, new_config):
        self.config = {**default_config, **new_config}
        self.verify_env()
        logger.info("Connecting to Kafka Producer bootstrap server")
        self.producer_client = KafkaProducer(bootstrap_servers=self.config.get("KAFKA_TARGET_BOOTSTRAP_SERVERS").split(","),
                                             value_serializer=lambda v: json.dumps(v).encode('utf-8'))
        self.target_topic = self.config.get("KAFKA_TARGET_TOPIC")

    def verify_env(self):
        verification_failed = False
        for key, value in self.config.items():
            if not str.strip(value):
                verification_failed = True
                logger.error(f"Environment variable '{key}' not set.")
        if verification_failed:
            quit(1)

    def produce_to_topic(self, data):
        self.producer_client.send(self.target_topic, data)

    def log_error(self, data):
        logger.error(f"In sending record: {data}")

    def restart_collector(self, new_config):
        self.__init__(self, new_config)



from __future__ import absolute_import

__title__ = 'procstream'

from procstream.process import StreamProcessMicroService
from procstream.source import DataSourceService
from procstream.source import TwitterDataCollector
from procstream.sink import DataSinkService

__all__ = ['StreamProcessMicroService', 'DataSourceService', 'DataSinkService','TwitterDataCollector']
